@@ -8,11 +8,22 @@
 #ifndef OPENCV_GAPI_OWN_TYPES_HPP
 #define OPENCV_GAPI_OWN_TYPES_HPP
 
-#   if 0
+#   if defined(__OPENCV_BUILD)
 #       include <opencv2/core/base.hpp>
 #       define GAPI_EXPORTS CV_EXPORTS
-
+        /* special informative macros for wrapper generators */
+#       define GAPI_PROP CV_PROP
+#       define GAPI_WRAP CV_WRAP
+#       define GAPI_EXPORTS_W_SIMPLE CV_EXPORTS_W_SIMPLE
+#       define GAPI_EXPORTS_W CV_EXPORTS_W
 #   else
+#       define GAPI_PROP
+#       define GAPI_WRAP
+#       define GAPI_EXPORTS
+#       define GAPI_EXPORTS_W_SIMPLE
+#       define GAPI_EXPORTS_W
+
+#if 0  // Note: the following version currently is not needed for non-OpenCV build
 #       if defined _WIN32
 #           define GAPI_EXPORTS __declspec(dllexport)
 #       elif defined __GNUC__ && __GNUC__ >= 4
@@ -22,6 +33,7 @@
 #       ifndef GAPI_EXPORTS
 #           define GAPI_EXPORTS
 #       endif
+#endif
 
 #   endif
 
